@@ -1,6 +1,5 @@
 --------------------------------------------------------------------------------
--- | This module exposes connection internals and should only be used if you
--- really know what you are doing.
+-- | This module exposes connection internals
 {-# LANGUAGE OverloadedStrings #-}
 module Network.WebSockets.Connection
     ( PendingConnection (..)
@@ -31,6 +30,7 @@ module Network.WebSockets.Connection
     , sendClose
     , sendCloseCode
     , sendPing
+    , sendPong
 
     , withPingThread
     , forkPingThread
@@ -49,6 +49,7 @@ import           Control.Applicative                             ((<$>))
 import           Control.Concurrent                              (forkIO,
                                                                   threadDelay)
 import qualified Control.Concurrent.Async                        as Async
+import           Control.Concurrent.MVar                         (MVar, newEmptyMVar, tryPutMVar)
 import           Control.Exception                               (AsyncException,
                                                                   fromException,
                                                                   handle,
@@ -178,13 +179,15 @@ acceptRequestWith pc ar = case find (flip compatible request) protocols of
         write <- foldM (\x ext -> extWrite ext x) writeRaw exts
         parse <- foldM (\x ext -> extParse ext x) parseRaw exts
 
-        sentRef    <- newIORef False
+        sentRef <- newIORef False
+        heartbeat <- newEmptyMVar
         let connection = Connection
                 { connectionOptions   = options
                 , connectionType      = ServerConnection
                 , connectionProtocol  = protocol
                 , connectionParse     = parse
                 , connectionWrite     = write
+                , connectionHeartbeat = heartbeat
                 , connectionSentClose = sentRef
                 }
 
@@ -238,6 +241,7 @@ rejectRequestWith pc reject = sendResponse pc $ Response
 
 
 --------------------------------------------------------------------------------
+-- | Requires calling 'pendingStream' and 'Stream.close'.
 rejectRequest
     :: PendingConnection  -- ^ Connection to reject
     -> B.ByteString       -- ^ Rejection response body
@@ -251,6 +255,9 @@ data Connection = Connection
     { connectionOptions   :: !ConnectionOptions
     , connectionType      :: !ConnectionType
     , connectionProtocol  :: !Protocol
+    , connectionHeartbeat :: !(MVar ())
+    -- ^ This MVar is filled whenever a pong is received.  This is used by
+    -- 'withPingPong' to timeout the connection if a pong is not received.
     , connectionParse     :: !(IO (Maybe Message))
     , connectionWrite     :: !([Message] -> IO ())
     , connectionSentClose :: !(IORef Bool)
@@ -293,6 +300,7 @@ receiveDataMessage conn = do
                 unless hasSentClose $ send conn msg
                 throwIO $ CloseRequest i closeMsg
             Pong _    -> do
+                _ <- tryPutMVar (connectionHeartbeat conn) ()
                 connectionOnPong (connectionOptions conn)
                 receiveDataMessage conn
             Ping pl   -> do
@@ -388,6 +396,10 @@ sendCloseCode conn code =
 sendPing :: WebSocketsData a => Connection -> a -> IO ()
 sendPing conn = send conn . ControlMessage . Ping . toLazyByteString
 
+--------------------------------------------------------------------------------
+-- | Send a pong
+sendPong :: WebSocketsData a => Connection -> a -> IO ()
+sendPong conn = send conn . ControlMessage . Pong . toLazyByteString
 
 --------------------------------------------------------------------------------
 -- | Forks a ping thread, sending a ping message every @n@ seconds over the
@@ -396,6 +408,9 @@ sendPing conn = send conn . ControlMessage . Ping . toLazyByteString
 -- This is useful to keep idle connections open through proxies and whatnot.
 -- Many (but not all) proxies have a 60 second default timeout, so based on that
 -- sending a ping every 30 seconds is a good idea.
+--
+-- Note that usually you want to use 'Network.WebSockets.Connection.PingPong.withPingPong'
+-- to timeout the connection if a pong is not received.
 withPingThread
     :: Connection
     -> Int    -- ^ Second interval in which pings should be sent.
@@ -405,7 +420,6 @@ withPingThread
 withPingThread conn n action app =
     Async.withAsync (pingThread conn n action) (\_ -> app)
 
-
 --------------------------------------------------------------------------------
 -- | DEPRECATED: Use 'withPingThread' instead.
 --
