@@ -22,6 +22,7 @@ tests :: Test
 tests = testGroup "Network.WebSockets.Http.Tests"
     [ testCase "jwebsockets response" jWebSocketsResponse
     , testCase "chromium response"    chromiumResponse
+    , testCase "matchbook response"   matchbookResponse
     ]
 
 
@@ -59,3 +60,29 @@ chromiumResponse = assert $ case A.parseOnly decodeResponseHead input of
         , ""
         , "No such target id: 20_1"
         ]
+
+--------------------------------------------------------------------------------
+-- | This is a specific response sent by Matchbook.com which caused trouble
+
+matchbookResponse :: Assertion
+matchbookResponse = assert $ case A.parseOnly decodeResponseHead input of
+    Left err -> error err
+    Right _  -> True
+  where
+    input = BC.intercalate "\r\n"
+        [ "HTTP/1.1 101 "
+        , "Date: Mon, 22 May 2017 19:39:08 GMT"
+        , "Connection: upgrade"
+        , "Set-Cookie: __cfduid=deadbeefdeadbeefdeadbeefdeadbeefdeadbeefdea; expires=Tue, 22-May-18 19:39:08 GMT; path=/; domain=.matchbook.com; HttpOnly"
+        , "X-Content-Type-Options: nosniff"
+        , "X-XSS-Protection: 1; mode=block"
+        , "X-Frame-Options: DENY"
+        , "Upgrade: websocket"
+        , "Sec-WebSocket-Accept: dEadB33fDeadbEEfD3aDbE3Fdea="
+        , "X-MB-HA: edge-socket"
+        , "X-MB-HAP: haproxy01aws"
+        , "Server: cloudflare-nginx"
+        , "CF-RAY: 3632deadbeef5b33-HEL"
+        , ""
+        , ""
+        ]
